# Importing packages

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import math

# Read the CSV file
amzn_data = pd.read_csv("AAPL10Years.csv")

KValuePeriod = 14
DValuePeriod = 3
SlowLength = 26
FastLength = 12
SignalPeriod = 9

close_prices = amzn_data[amzn_data.columns[4]]
low_prices = amzn_data[amzn_data.columns[3]]
high_prices = amzn_data[amzn_data.columns[2]]
close_prices_year = close_prices[KValuePeriod - 1:]

KValues = []
for x in range(KValuePeriod, (len(close_prices) + 1)):
    LowOverPeriod = min(low_prices[x - KValuePeriod:x])
    HighOverPeriod = max(high_prices[x - KValuePeriod:x])
    KValues.append(100 * ((close_prices[x - 1] - LowOverPeriod) / (HighOverPeriod - LowOverPeriod)))
    
DValues = []
for x in range(DValuePeriod, (len(KValues) + 1)):
    SumOfPeriod = sum(KValues[x - DValuePeriod:x])
    DValues.append(SumOfPeriod / DValuePeriod)

SlowEMA = []
for x in range(SlowLength, len(close_prices) + 1):
    SumOfPeriod = sum(close_prices[x - SlowLength:x])
    SlowEMA.append(SumOfPeriod / SlowLength)

FastEMA = []
for x in range(FastLength, (len(close_prices) + 1)):
    SumOfPeriod = sum(close_prices[x - FastLength:x])
    FastEMA.append(SumOfPeriod / FastLength)

FastEMA = FastEMA[(len(FastEMA) - len(SlowEMA)):]

MACD = []
for x in range(0, len(SlowEMA)):
    MACD.append(FastEMA[x] - SlowEMA[x])

SignalLine = []
for x in range(SignalPeriod, (len(MACD) + 1)):
    SumOfPeriod = sum(MACD[x - SignalPeriod:x])
    SignalLine.append(SumOfPeriod / SignalPeriod)

MACD = MACD[(len(MACD) - len(SignalLine)):]
Histogram = []
for x in range(0, len(MACD)):
    Histogram.append(MACD[x] - SignalLine[x])

x_axis = np.linspace(1, len(close_prices_year), num=len(close_prices_year))

plt.figure(figsize=(16,6))
plt.subplot(3, 1, 1)
plt.subplots_adjust(hspace=1)
# set the spacing between subplots
plt.plot(x_axis, close_prices_year, color='r', linewidth=0.75)
plt.title('AMZN Stock [Close Prices] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Close Price (USD)')




xaxisKValues = np.linspace(1, (len(KValues) - DValuePeriod), num=(len(KValues) - DValuePeriod))
xaxisDValues = np.linspace(1, len(DValues), num=len(DValues))
plt.subplot(3,1,2)
plt.subplots_adjust(hspace=1)
plt.plot(xaxisKValues, KValues[DValuePeriod:], color='r', linewidth=0.75, label="%K Line")
plt.plot(xaxisDValues, DValues, color='b', linewidth=0.75, label="%D Line")
plt.title('AMZN Stock [Stochastic Oscillator] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Normalised Value')
plt.axhline(y = 70, color = 'black', linestyle = '--', linewidth=0.75, label="Thresholds")
plt.axhline(y = 30, color = 'black', linestyle = '--', linewidth=0.75)
plt.legend(loc='upper left')




xaxis = np.linspace(1, (len(MACD)), num=(len(MACD)))
plt.subplot(3,1,3)
plt.subplots_adjust(hspace=1)
plt.plot(xaxis, MACD, color='black', linewidth=0.75, label="MACD Line")
plt.plot(xaxis, SignalLine, color='b', linewidth=0.75, label="Signal Line")

for x in range(0, len(Histogram)):
    if (Histogram[x] >= 0):
        Col = "green"
    else:
        Col = "red"
    rect=mpatches.Rectangle((x,0),1,Histogram[x], 
                            fill = True,
                            color = Col,
                            linewidth = 2)
    plt.gca().add_patch(rect)

plt.title('AMZN Stock [MACD] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Value')

plt.legend(loc='lower right')


plt.show()



# Trading Strategy

def Implement_STO_MACD_Strategy(close, k, d, macd, macd_signal):
    buy_price = []
    sell_price = []
    sto_macd_signal = []
    signal = 0
    
    minIndex = min([len(close), len(k), len(d), len(macd), len(macd_signal)])
    k = k[(len(k) - minIndex):]
    d = d[(len(d) - minIndex):]
    close = list(close[(len(close) - minIndex):])
    macd = macd[(len(macd) - minIndex):]
    macd_signal = macd_signal[(len(macd_signal) - minIndex):]

    for x in range(len(close)):
        if k[x] < 30 and d[x] < 30 and macd[x] < -2 and macd_signal[x] < -2:
            if signal != 1:
                buy_price.append(close[x])
                sell_price.append(np.nan)
                signal = 1
                sto_macd_signal.append(signal)
            else:
                buy_price.append(np.nan)
                sell_price.append(np.nan)
                sto_macd_signal.append(0)
        elif k[x] > 70 and d[x] > 70 and macd[x] > 2 and macd_signal[x] > 2:
            if signal != -1 and signal != 0:
                buy_price.append(np.nan)
                sell_price.append(close[x])
                signal = -1
                sto_macd_signal.append(signal)
            else:
                buy_price.append(np.nan)
                sell_price.append(np.nan)
                sto_macd_signal.append(0)
        else:
            buy_price.append(np.nan)
            sell_price.append(np.nan)
            sto_macd_signal.append(0)
    return buy_price, sell_price, sto_macd_signal, close

buy_price, sell_price, sto_macd_signal, close_prices = Implement_STO_MACD_Strategy(close_prices, KValues, DValues, MACD, SignalLine)

InitialInvestment = 100000
NumberOfStocks = 0

for x in range(len(sto_macd_signal)):
    if (sto_macd_signal[x] == 1):
        NumberOfStocks = math.floor(InitialInvestment / buy_price[x])
        InitialInvestment -= buy_price[x]*NumberOfStocks
        print("Buy Price: " + buy_price[x])
        print("New Total: " + str(round(InitialInvestment, 2)))
    elif (sto_macd_signal[x] == -1):
        InitialInvestment += sell_price[x]*NumberOfStocks
        NumberOfStocks = 0
        print("Sell Price: " + sell_price[x])
        print("New Total: " + str(round(InitialInvestment, 2)))
print("Return: " + str(round(InitialInvestment, 2)))
        


