import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches

# Read the CSV file
amzn_data = pd.read_csv("AAPL1Year.csv")

SlowLength = 26
FastLength = 12
SignalPeriod = 9

close_prices = amzn_data[amzn_data.columns[4]]
low_prices = amzn_data[amzn_data.columns[3]]
high_prices = amzn_data[amzn_data.columns[2]]
close_prices_year = close_prices[SlowLength - 1:]

x_axis = np.linspace(1, len(close_prices_year), num=len(close_prices_year))

plt.figure(figsize=(16,6))
plt.subplot(2, 1, 1)
plt.subplots_adjust(hspace=0.5)
# set the spacing between subplots
plt.plot(x_axis, close_prices_year, color='r', linewidth=0.75)
plt.title('AAPL Stock [Close Prices] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Close Price (USD)')

SlowEMA = []
for x in range(SlowLength, len(close_prices) + 1):
    SumOfPeriod = sum(close_prices[x - SlowLength:x])
    SlowEMA.append(SumOfPeriod / SlowLength)

FastEMA = []
for x in range(FastLength, (len(close_prices) + 1)):
    SumOfPeriod = sum(close_prices[x - FastLength:x])
    FastEMA.append(SumOfPeriod / FastLength)

FastEMA = FastEMA[(len(FastEMA) - len(SlowEMA)):]

MACD = []
for x in range(0, len(FastEMA)):
    MACD.append(FastEMA[x] - SlowEMA[x])

SignalLine = []
for x in range(SignalPeriod, (len(MACD) + 1)):
    SumOfPeriod = sum(MACD[x - SignalPeriod:x])
    SignalLine.append(SumOfPeriod / SignalPeriod)

MACD = MACD[(len(MACD) - len(SignalLine)):]
Histogram = []
for x in range(0, len(MACD)):
    Histogram.append(MACD[x] - SignalLine[x])

xaxis = np.linspace(1, (len(MACD)), num=(len(MACD)))
plt.subplot(2,1,2)
plt.plot(xaxis, MACD, color='black', linewidth=0.75, label="MACD Line")
plt.plot(xaxis, SignalLine, color='b', linewidth=0.75, label="Signal Line")

for x in range(0, len(Histogram)):
    if (Histogram[x] >= 0):
        Col = "green"
    else:
        Col = "red"
    rect=mpatches.Rectangle((x,0),1,Histogram[x], 
                            fill = True,
                            color = Col,
                            linewidth = 2)
    plt.gca().add_patch(rect)

plt.title('AAPL Stock [MACD] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Value')

plt.legend(loc='lower right')
plt.show()




