import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

# Read the CSV file
amzn_data = pd.read_csv("AAPL1Year.csv")

KValuePeriod = 14
DValuePeriod = 3

close_prices = amzn_data[amzn_data.columns[4]]
low_prices = amzn_data[amzn_data.columns[3]]
high_prices = amzn_data[amzn_data.columns[2]]
close_prices_year = close_prices[KValuePeriod - 1:]

x_axis = np.linspace(1, len(close_prices_year), num=len(close_prices_year))

plt.figure(figsize=(16,6))
plt.subplot(2, 1, 1)
plt.subplots_adjust(hspace=0.5)
# set the spacing between subplots
plt.plot(x_axis, close_prices_year, color='r', linewidth=0.75)
plt.title('AAPL Stock [Close Prices] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Close Price (USD)')

KValues = []
for x in range(0, (len(close_prices) - KValuePeriod + 1)):
    LowOverPeriod = min(low_prices[x:x + KValuePeriod])
    HighOverPeriod = max(high_prices[x:x + KValuePeriod])
    KValues.append(100 * ((close_prices[x + KValuePeriod - 1] - LowOverPeriod) / (HighOverPeriod - LowOverPeriod)))
    
DValues = []
for x in range(DValuePeriod, (len(KValues))):
    SumOfPeriod = sum(KValues[x - DValuePeriod:x])
    DValues.append(SumOfPeriod / DValuePeriod)

xaxisKValues = np.linspace(1, (len(KValues) - DValuePeriod), num=(len(KValues) - DValuePeriod))
xaxisDValues = np.linspace(1, len(DValues), num=len(DValues))
plt.subplot(2,1,2)
plt.plot(xaxisKValues, KValues[DValuePeriod:], color='r', linewidth=0.75, label="%K Line")
plt.plot(xaxisDValues, DValues, color='b', linewidth=0.75, label="%D Line")
plt.title('AAPL Stock [Stochastic Oscillator] 18-08-22 to 18-08-23')
plt.xlabel('Time Periods (Days)')
plt.ylabel('Normalised Value')
plt.axhline(y = 70, color = 'black', linestyle = '--', linewidth=0.75, label="Thresholds")
plt.axhline(y = 30, color = 'black', linestyle = '--', linewidth=0.75)
plt.legend(loc='upper left')
plt.show()
        

